import csv
import logging
from pathlib import Path

from ldappy.core.lib.ldapur import LdapUr
from pydantic import BaseModel, Field

logger = logging.getLogger(__name__)


class Employee(BaseModel):
    uid: str
    nom: str
    prenom: str
    mail: str = Field(pattern=r"^[a-zA-Z0-9._%+-]+@univ-reunion\.fr$")
    affectation: str = Field(min_length=10, max_length=10, serialization_alias="code_parent")


class ListGpucEmployee(BaseModel):
    employee: list[Employee]

    @classmethod
    def build_ldappy(cls) -> "ListGpucEmployee":
        """Construit une instance de ListGpucEmployee à partir des données provenant de LDAP via ldappy.

        Returns:
            ListGpucEmployee: Instance construite avec les données de LDAP.
        """
        # Récupération des données d'employee via le ldap
        connecteur = LdapUr()
        response = connecteur.get_employee()
        connecteur.unbind()

        data = []

        for tmp in response:
            employee = Employee(
                uid=tmp["attributes"]["supannAliasLogin"][0],
                nom=tmp["attributes"]["runUnivNom"][0],
                prenom=tmp["attributes"]["runUnivPrenom"][0],
                mail=tmp["attributes"]["mail"][0],
                affectation=tmp["attributes"]["supannEntiteAffectationPrincipale"][0],
            )
            data.append(employee)

        return cls(employee=data)

    def to_csv(self, filename: str):
        keys = self.employee[0].model_dump(by_alias=True).keys()
        with Path(filename).open(mode="w", encoding="utf8", newline="") as output_file:
            dict_writer = csv.DictWriter(output_file, keys)
            dict_writer.writeheader()
            for empl in self.employee:
                dict_writer.writerow(empl.model_dump(by_alias=True))
        logger.debug("Sauvegarde du fichier %s", filename)
