import csv
import logging
import re
from pathlib import Path

from graphviz import Digraph
from ldappy.core.lib.ldapur import LdapUr
from pydantic import BaseModel, Field
from wssiham.core.dossier_parametrage.recupStructures import RecupStructures

logger = logging.getLogger(__name__)


class Structure(BaseModel):
    code_uo: str = Field(min_length=10, max_length=10, serialization_alias="codeUO")
    lib_uo: str = Field(serialization_alias="libUO")
    niveau: int = Field(default=0, exclude=True)
    code_uo_parent: str = Field(default="", serialization_alias="codeUOParent")
    uid_responsable: str = Field(default="", serialization_alias="uidResponsable")


class ListGpucStructure(BaseModel):
    structure: list[Structure]

    @classmethod
    def build_ws(cls) -> "ListGpucStructure":
        """Construit une instance de ListGpucStructure à partir des données provenant de l'API WS (Web Service).

        Returns:
            ListGpucStructure: Instance construite avec les données de l'API WS.
        """

        def parcours_structure(unite, niveau: int = 0, unite_parent: str = ""):
            response = (
                RecupStructures(liste_uo=[{"codeUO": unite}], code_administration="RUN")
                .response()
                .uniteOrganisationnelle[0]
            )

            structure = Structure(
                code_uo=response.codeUO,
                lib_uo=response.libLongUO,
                niveau=niveau,
                code_uo_parent=unite_parent,
                uid_responsable="",
            )

            data.append(structure)

            # Parcours des unités filles
            for unite_fille in response.unitesFilles:
                if unite_fille.codeTypeArborescence == "HIE" and not any(
                    d.code_uo == unite_fille.codeUOfille for d in data
                ):
                    parcours_structure(unite_fille.codeUOfille, niveau + 1, unite)

        data = []
        parcours_structure("HU00000000")
        return cls(structure=data)

    @classmethod
    def build_ldappy(cls) -> "ListGpucStructure":
        """Construit une instance de ListGpucStructure à partir des données provenant de LDAP via ldappy.

        Returns:
            ListGpucStructure: Instance construite avec les données de LDAP.
        """

        # DEPRECATED 13/03/2024
        def lendepth(code: str) -> int:
            """Récupère la profondeur d'une structure à partir de son code UO

            Warnings:
                Ce code ne fonctionnera pas lors de la prochaine mise à jour des structures dans SIHAM

            Args:
                code (str): Le code UO de la structure (code sur 10 caractères alphanumérique)

            Returns:
                Un entier qui détermine la profondeur (le niveau) d'une UO
            """
            # Séquence de découpe du code
            sequence = [2, 1, 3, 2, 2]
            scode = []
            index = 0

            # Construction des parties de code en fonction de la séquence
            for s in sequence:
                partie = code[index : index + s]
                scode.append(partie)
                index += s

            depth = -1

            # Parcours de la liste de code
            for p in scode:
                # Si tous les caractères de la liste ne sont pas null
                if not all(letter == "0" for letter in p):
                    # Ajout 1 à la profondeur
                    depth += 1
                # Sinon, on arrête le calcul
                else:
                    break
            return depth

        def lenchem(infuo: str) -> int:
            """Récupère la profondeur d'une structure à partir de son code UO

            Args:
                infuo (str): L'information disponible dans l'attribut runUnivStrInfos de l'UO dans Ldap

            Returns:
                Un entier qui détermine la profondeur (le niveau) d'une UO

            Examples:
                >>> a = "[source={SIHAM}HUX0000000][chemin=Université/Extérieur]"
                >>> b = a + "[date_deb=19500101000000+0400][date_fin=29991231000000+0400][date_rnv=]"
                >>> lenchem(b)
                1
            """
            # Motif regex pour la valeur de la clé "chemin"
            pattern = r"\[chemin=(.*?)\]"

            # Recherche de la correspondance dans le string
            match = re.search(pattern, infuo)

            # Vérification si une correspondance a été trouvée
            if match:
                # Récupération de la valeur de la clé "chemin"
                chemin_value = match.group(1)
                # Affichage de la valeur
                return len(chemin_value.split("/")) - 1
            return -1

        # Récupération des données de structures via le ldap
        connecteur = LdapUr()
        uos = connecteur.get_siham_alluo()
        connecteur.unbind()

        data = []

        for uo in uos:
            # code uo
            vcodeuo = uo["attributes"]["supannCodeEntite"][0]

            # code parent uo
            codeuop = uo["attributes"]["supannCodeEntiteParent"]

            # chemin structure
            chemstruct = uo["attributes"]["runUnivStrInfos"][0]

            structure = Structure(
                code_uo=vcodeuo,
                niveau=lenchem(chemstruct),
                lib_uo=uo["attributes"]["description"][0],
                code_uo_parent=codeuop[0] if codeuop else "",
                uid_responsable="",
            )

            data.append(structure)

        return cls(structure=data)

    def _generate_csv(self, filename: str, fieldnames: list[str], level: int):
        """
        Génère un fichier CSV avec les données correspondant à un niveau de la structure.

        Args:
            filename (str): Nom du fichier CSV à générer.
            fieldnames (list): Liste des noms de champs pour l'en-tête du fichier CSV.
            level (int): Niveau de la structure.

        """
        # Récupère toutes les structures sous forme de liste en fonction du niveau recherché
        tmp = [x for x in self.structure if x.niveau == level]

        # Récupère le nom des attributs de l'objet structure
        keys = list(self.structure[0].model_dump(by_alias=True).keys())

        # Si le niveau est à 1, on retire les codes uo parent pour correspondre au template
        if level == 1:
            keys.remove("codeUOParent")

        with Path(filename).open(mode="w", encoding="utf8", newline="") as csv_file:
            csv_writer = csv.DictWriter(csv_file, fieldnames=fieldnames, delimiter=";")

            # Écrire l'en-tête du fichier CSV
            csv_writer.writeheader()

            # Écrire les données dans le fichier CSV
            for row in tmp:
                csv_writer.writerow({f: row.model_dump(by_alias=True).get(k, "") for f, k in zip(fieldnames, keys)})

    def to_csv_level(self, filename: str, niveau: int):
        """Génère un fichier CSV pour un niveau de structure donnée

        Args:
            filename (str): nom du fichier csv
            niveau (int): profondeur de l'arbre
        """
        if niveau == 1:
            fieldnames = ["CODE_AFFECTATION", "NOM_AFFECTATION", "UID_DIRECTEUR"]
        elif niveau == 2:
            fieldnames = ["CODE_DEPARTEMENT", "NOM_DEPARTEMENT", "CODE_AFFECTATION", "UID_DIRECTEUR"]
        elif niveau == 3:
            fieldnames = ["CDOE_EQUIPE", "NOM_EQUIPE", "CODE_PARENT", "UID_DIRECTEUR"]
        elif niveau == 4:
            fieldnames = ["CODE_DIVISION", "NOM_DIVISION", "CODE_EQUIPE", "UID_DIRECTEUR"]
        elif niveau == 5:
            fieldnames = ["CODE_SOUS_DIVISION", "NOM_SOUS_DIVISION", "CODE_DIVISION", "UID_DIRECTEUR"]
        else:
            excep = f"Le niveau doit être compris entre 1 et 4 (inclus) - Retour {niveau}"
            raise Exception(excep)

        self._generate_csv(filename, fieldnames, niveau)
        logger.debug("Sauvegarde du csv %s", filename)

    def generate_graph(self, filename: str) -> None:
        """Génère un graphique représentant les liens entre les structures

        Args:
            filename (str): Nom du fichier de sortie
        """
        # Initialisation d'un graphique orienté
        graph = Digraph(format="svg", node_attr={"shape": "box"}, graph_attr={"rankdir": "LR"})

        # Ajout des nœuds au graphique
        for struct in self.structure:
            # Utilisation du codeUO comme identifiant unique pour chaque nœud
            graph.node(struct.code_uo, label=f"{struct.code_uo}\n{struct.lib_uo}")

        # Ajout des liens entre les nœuds (parent-enfant)
        for struct in self.structure:
            # Si la structure a un parent, ajouter un lien entre eux
            if struct.code_uo_parent:
                graph.edge(struct.code_uo_parent, struct.code_uo)

        # Sauvegarde du graphique dans un fichier
        graph_file = f"{filename}"
        graph.render(graph_file, cleanup=True)
