import logging
from datetime import datetime, timedelta
from typing import Optional

from pydantic import BaseModel, Field, ValidationError
from pyetl.model.ohris.agent.type import ContratType, GenreType, GroupeType, QuotiteType
from pyetl.model.ohris.etiquette import ModaliteService
from pyetl.model.ohris.uxml import Utilisateurs
from pyetl.model.siham.status import SousCategorie, TypeDroit
from pyetl.tools.toolbox.mxml import Mxml
from siham_api_client.models.ohris_agent import OhrisAgent
from siham_api_client.models.service_mode import ServiceMode

logger = logging.getLogger(__name__)


class Compte(BaseModel):
    """Représentation de la structure des données agents pour le flux utilisateurs Ohris"""

    matricule: str
    nom: str
    prenom: str
    genre: GenreType
    login: str
    mail: str
    type_contrat: ContratType
    groupe: GroupeType
    libelle_long_uo: str
    code_uo: str
    date_debut_contrat: str
    date_fin_contrat: str
    valideur: Optional[str] = Field(default="")
    quotite: Optional[QuotiteType] = Field(default="100")
    date_affectation_quotite: str
    enfants: Optional[list[str]]

    @staticmethod
    def _cs_to_tc(code_tc: str) -> str:
        """Transforme le code statut (code pip) en code type contrat ohris

        Args:
            code_tc (str): code type contrat

        Returns:
            L'équivalent du code type contrat ohris pour le code statut (code pip SIHAM) de l'agent
        """
        if code_tc == "TITU1":
            return "Titulaire"
        if code_tc == "STAGI1":
            return "Stagiaire"
        if code_tc.startswith("HB"):
            return "Hébergé"
        if code_tc.startswith("C"):
            return "Contractuel"
        return "Hébergé"

    @staticmethod
    def _get_contrat_date(contrat_date: list[tuple[datetime, datetime]]) -> tuple[datetime, datetime]:
        """Récupère la 1ère date de début sans discontinuité et la dernière date de fin sans discontinuité

        Args:
            contrat_date (list[tuple]): Une liste de date (date de début, date de fin) au format string

        Returns:
            Un tuple avec la 1ère date de début et la dernière date de fin sans discontinuité

        Exemples:
            >>> import datetime
            >>> l = [(datetime.datetime(2024, 1, 1, 0, 0), datetime.datetime(2024, 8, 31, 0, 0)), \
            (datetime.datetime(2021, 9, 1, 0, 0), datetime.datetime(2022, 8, 31, 0, 0)), \
            (datetime.datetime(2023, 9, 1, 0, 0), datetime.datetime(2023, 12, 31, 0, 0)), \
            (datetime.datetime(2022, 9, 1, 0, 0), datetime.datetime(2023, 8, 31, 0, 0))]
            >>> result = Compte._get_contrat_date(l)
            >>> print(result)
            (datetime.datetime(2021, 9, 1, 0, 0), datetime.datetime(2024, 8, 31, 0, 0))

            >>> m = [(datetime.datetime(2024, 1, 1, 0, 0), datetime.datetime(2024, 12, 31, 0, 0)), \
            (datetime.datetime(2022, 9, 1, 0, 0), datetime.datetime(2023, 8, 31, 0, 0))]
            >>> r = Compte._get_contrat_date(m)
            >>> print(r)
            (datetime.datetime(2024, 1, 1, 0, 0), datetime.datetime(2024, 12, 31, 0, 0))

        """
        # Convertir les chaînes de date en objets datetime
        dates_debut = [start for start, _ in contrat_date]
        dates_fin = [end for _, end in contrat_date]

        # Trier toutes les dates de début
        dates_debut.sort()
        dates_fin.sort()

        # Initialiser les variables pour la première date de début
        premiere_date_debut = None
        # derniere_date_fin = None

        # Parcourir les dates pour trouver la première date de début et la dernière date de fin avec continuité
        for i in range(len(dates_debut) - 1):
            if dates_fin[i] + timedelta(days=1) == dates_debut[i + 1]:
                if premiere_date_debut is None:
                    premiere_date_debut = dates_debut[i]
                # derniere_date_fin = dates_fin[i + 1]

        # Utiliser la dernière date de début et de fin dans la liste si la continuité n'est pas trouvée
        if premiere_date_debut is None and len(dates_debut) > 0:
            premiere_date_debut = dates_debut[-1]
            # derniere_date_fin = dates_fin[-1]

        return premiere_date_debut, dates_fin[-1]

    @staticmethod
    def _get_latest_assignment(data: OhrisAgent.assignments) -> tuple[Optional[str], Optional[str]]:
        """Récupère la dernière affectation de l'agent

        Args:
            data (OhrisAgent.assignments.additional_properties): Les données d'affectation de l'agent

        Returns:
            Le couple (code uo, libelle long uo) de la dernière affectation hierarchique de l'agent
        """
        latest_info = max(data.values(), key=lambda info: info.start, default=None)

        if latest_info is not None:
            organizational_unit = latest_info.organizational_unit
            code = organizational_unit.code
            longname = organizational_unit.longname
            return code, longname
        return None, None

    @staticmethod
    def _get_latest_quotite(data: list[ServiceMode]) -> tuple[Optional[str], Optional[datetime]]:
        """Récupère la dernière quotité de l'agent ainsi que la date de début associé à cette quotité

        Args:
            data (list[ServiceMode]): Une liste des modalités de service de l'agent

        Returns:
            Un tuple contenant la valeur du code de la dernière quotite de l'agent ainsi que la date de début associé

        """
        latest_info = max(data, key=lambda info: info.start, default=None)

        if latest_info is not None:
            mode = latest_info.mode
            code = mode.code
            last_date = latest_info.start
            return ModaliteService[code].value, last_date
        return None, None

    @staticmethod
    def _set_to_groupe(code_status: str, first_date: datetime, last_date: datetime) -> str:
        # Calcul de la durée entre les deux dates
        diff = last_date - first_date

        # Comparaison avec une durée d'un an (365 jours)
        one_year = timedelta(days=365)

        # Retourne True si la différence est supérieure à un an, sinon False
        if code_status.startswith("HB"):
            return GroupeType.HEBERGE.value
        if diff < one_year:
            return GroupeType.CONTM1.value
        if code_status in TypeDroit.PRIVE.value:
            return GroupeType.CONTDP.value
        if code_status in SousCategorie.BIATSS.value:
            return GroupeType.BIATSS.value
        if code_status in SousCategorie.ENSEIGNANT_CHERCHEUR.value:
            return "ENSEIGNANT_CHERCHEUR"
        return GroupeType.BIATSS.value

    @classmethod
    def from_users_dict(cls, agent_ohris_dict: OhrisAgent) -> Optional["Compte"]:
        genre = {"1": "homme", "2": "femme", "3": "femme"}

        code_statut = agent_ohris_dict.statuses[0].code
        type_contrat = cls._cs_to_tc(code_statut)
        dates_list = [(item.start, item.end) for item in agent_ohris_dict.assignments.additional_properties.values()]

        ddc, dfc = cls._get_contrat_date(dates_list)

        groupe = cls._set_to_groupe(code_statut, ddc, dfc)

        code_uo, ll_uo = cls._get_latest_assignment(agent_ohris_dict.assignments.additional_properties)

        if agent_ohris_dict.children:
            chlidsbirthday = [
                childbirthday.birthday.strftime("%Y-%m-%d") for childbirthday in agent_ohris_dict.children
            ]
        else:
            chlidsbirthday = []

        quotite, date_quotite = cls._get_latest_quotite(agent_ohris_dict.service_modes)

        try:
            return cls(
                matricule=agent_ohris_dict.matricule,
                nom=agent_ohris_dict.usualname,
                prenom=agent_ohris_dict.usual_firstname,
                genre=genre[agent_ohris_dict.civility.code],
                login=agent_ohris_dict.login,
                mail=agent_ohris_dict.mail,
                type_contrat=type_contrat,
                date_debut_contrat=ddc.strftime("%Y-%m-%d"),
                date_fin_contrat=dfc.strftime("%Y-%m-%d"),
                groupe=groupe,
                libelle_long_uo=ll_uo,
                code_uo=code_uo,
                valideur=None,
                quotite=quotite,
                date_affectation_quotite=date_quotite.strftime("%Y-%m-%d"),
                enfants=chlidsbirthday,
            ).model_dump()
        except ValidationError:
            logger.exception("Erreur de validation sur le dossier : %s", agent_ohris_dict.matricule)


class ListCompte(BaseModel):
    """Liste d'objet 'Compte'"""

    agents_compte: list[Compte]

    def to_xml(self, namefile: str):
        usrs = [x.model_dump() for x in self.agents_compte]
        Mxml.sxml_to_file(
            Utilisateurs(utilisateur=usrs).to_xml(encoding="UTF-8", pretty_print=True), outputfile=namefile
        )
        logger.debug("Sauvegarde du fichier xml %s", namefile)
