import csv
import logging
from pathlib import Path
from typing import Optional

from pydantic import BaseModel, ConfigDict, Field
from pyetl.model.ohris.etiquette import CodeAbsOhris
from siham_api_client.models.absence import Absence

logger = logging.getLogger(__name__)


class Conge(BaseModel):
    """Objet conge qui va servir pour la construction du fichier csv 'Absence Paie'"""

    model_config = ConfigDict(populate_by_name=True)

    societe: str = Field(default="HU00000000", alias="Societe")
    matricule_siham: str = Field(alias="Matricule")
    code_abs_siham: str = Field(exclude=True)
    code_abs_ohris: Optional[CodeAbsOhris] = Field(default=None, alias="Code absence")
    date_debut: str = Field(alias="Date de début")  # YYYY-mm-dd

    # Absence débute : '0' = matin, '1' = aprem
    demi_journee_debut: str = Field(default="0", alias="Demi-journee debut")

    date_fin: str = Field(alias="Date de fin")  # YYYY-mm-dd

    # Absence termine : '0' = matin, '1' = aprem
    demi_journee_fin: str = Field(default="1", alias="Demi-journee fin")

    def model_post_init(self, ctx):
        # Si aucun code défini, on définit la valeur par défaut
        if not self.code_abs_ohris:
            self.code_abs_ohris = CodeAbsOhris.GENERIQUE.value

    @classmethod
    def from_absence_dict(cls, absence_dict: Absence) -> "Conge":
        """Construction de l'objet Conge à partir du modèle de donnée "Absence" fournis par "siham-api-client"

        Args:
            absence_dict (Absence): Dictionnaire généré à partir du modèle de donnée fournit par "siham-api-client"

        Returns:
            L'objet conge construit à partir d'un dictionnaire par siham-api-client
        """
        djd = "0"
        djf = "1"
        # Si la date de debut de l'absence commence l'après-midi
        if absence_dict.start_midday:
            djd = "1"

        # Si la de fin de l'absence se termine le matin
        if absence_dict.end_midday:
            djf = "0"

        return cls(
            matricule_siham=absence_dict.matricule,
            date_debut=absence_dict.start.strftime("%Y-%m-%d"),
            demi_journee_debut=djd,
            date_fin=absence_dict.end.strftime("%Y-%m-%d"),
            demi_journee_fin=djf,
            code_abs_siham=absence_dict.type.code,
        )


class ListConge(BaseModel):
    """Liste d'objet 'Conge'"""

    agents_absence: list[Conge]
    columns: list[str] = [
        "Societe",
        "Matricule",
        "Code absence",
        "Date de début",
        "Demi-journee debut",
        "Date de fin",
        "Demi-journee fin",
    ]

    def to_csv(self, file: str) -> None:
        """Sauvegarde du model en un fichier csv compatible avec le format de ohris

        Args:
            file (str): nom du fichier à sauvegarder

        """
        with Path(file).open(mode="w", newline="") as csv_file:
            csv_writer = csv.DictWriter(csv_file, fieldnames=self.columns, delimiter=";")

            # Écrire l'en-tête du fichier CSV
            csv_writer.writeheader()

            # Écrire chaque instance dans le fichier CSV
            for agent_abs in self.agents_absence:
                csv_writer.writerow(agent_abs.model_dump(by_alias=True))
            logger.debug("Sauvegarde du fichier csv %s", csv_file.name)
