import logging
from enum import Enum

from notifire.mail.mailsender import MailSender
from pyetl.config.global_settings import DATE_DU_JOUR, GPUC_LOG, MAIL, TMP
from pyetl.config.gpuc_settings import GPUC_EMPLOYEE, GPUC_MAIL_DEST, GPUC_STRUCTURE, SFTP_GPUC
from pyetl.model.gpuc.employee import ListGpucEmployee
from pyetl.model.gpuc.structure import ListGpucStructure
from pyetl.tools.sftp import SFTPClient

logger = logging.getLogger(__name__)


class DataSource(Enum):
    LDAP: str = "ldap"
    WSSIHAM: str = "webservice_siham"


class Gpuc:
    @staticmethod
    def _get_structure(tag: DataSource) -> ListGpucStructure:
        """Crée la liste des structures en fonction du tag fourni.
        Structure SIHAM ou structure LDAP

        Args:
            tag (DataSource): Enum DataSource avec comme valeur unique 'ldap' ou 'webservice_siham'

        Returns:
            La liste des structures en fonction de la source de donnée

        """
        if not isinstance(tag, DataSource):
            excep = "La source de donnée définie n'est pas disponible dans DataSource"
            raise TypeError(excep)

        if tag.value == "ldap":
            logger.debug("Récupération des données de structure via le ldap")
            return ListGpucStructure.build_ldappy()
        logger.debug("Récupération des données de structure via le Webservice de SIHAM")
        return ListGpucStructure.build_ws()

    @staticmethod
    def build_csv_structure(*, sftp: bool = False, tag: DataSource = DataSource.LDAP) -> bool:
        """Crée les fichiers csv structure de niveau 1, 2, 3 et 4.
        Les fichiers sont sauvegardés en local sur le serveur en backup et peuvent être envoyé par sftp sur un
        serveur distant

        Args:
            sftp (bool): Push ou non les fichiers générés
            tag (DataSource): Quel sera la source de donnée. Valeur par défaut 'ldap'

        Returns:
            Booléen. True, les fichiers ont bien été généré. False sinon
        """
        gpstruct = Gpuc._get_structure(tag)

        if gpstruct:
            niveaux = [1, 2, 3, 4, 5]
            namefile = "affectation_niveau"

            for niveau in niveaux:
                logger.debug("Création de la structure de niveau %s", niveau)
                gpstruct.to_csv_level(f"{GPUC_STRUCTURE}/{namefile}{niveau}_{DATE_DU_JOUR}.csv", niveau)

            if sftp:
                # ... envoie des fichiers sur le serveur sftp
                sftp_client = SFTPClient(
                    SFTP_GPUC["SFTP_HOST"],
                    SFTP_GPUC["SFTP_PORT"],
                    SFTP_GPUC["SFTP_USERNAME"],
                    SFTP_GPUC["SFTP_PASSWORD"],
                    SFTP_GPUC["SSH_PPK"],
                )
                sftp_client.connect()
                for niveau in niveaux:
                    sftp_client.upload_file(
                        f'{SFTP_GPUC["PATH_FPROD"]}/{namefile}{niveau}.csv',
                        f"{GPUC_STRUCTURE}/{namefile}{niveau}_{DATE_DU_JOUR}.csv",
                    )
                    logger.debug("Envoie du fichier %s%s.csv via sftp", namefile, niveau)
                sftp_client.disconnect()
            return True

        return False

    @staticmethod
    def build_csv_employee(*, sftp: bool = False, notification: bool = False) -> bool:
        """Crée le fichier csv pour les employés
        Le fichier est sauvegardé en local sur le serveur en backup et peuvent être envoyé par sftp sur un
        serveur distant

        Args:
            sftp (bool): Push ou non le fichier généré
            notification (bool): Active ou non la notification mail

        Returns:
            Booléen. True, le fichier a bien été généré. False sinon
        """
        logger.debug("Récupération des données employées via le Ldap")
        gpemployee = ListGpucEmployee.build_ldappy()

        if gpemployee:
            namefile = "personnel"
            csvfile = f"{GPUC_EMPLOYEE}/{namefile}_{DATE_DU_JOUR}.csv"

            logger.debug("Création du csv comprenant les employées")
            gpemployee.to_csv(csvfile)

            if sftp:
                # ... envoie des fichiers sur le serveur sftp
                sftp_client = SFTPClient(
                    SFTP_GPUC["SFTP_HOST"],
                    SFTP_GPUC["SFTP_PORT"],
                    SFTP_GPUC["SFTP_USERNAME"],
                    SFTP_GPUC["SFTP_PASSWORD"],
                    SFTP_GPUC["SSH_PPK"],
                )
                sftp_client.connect()
                sftp_client.upload_file(f'{SFTP_GPUC["PATH_FPROD"]}/{namefile}.csv', csvfile)
                sftp_client.disconnect()
                logger.debug("Envoie du fichier %s.csv via sftp", namefile)

                # Suppression du fichier généré après envoie par souci de sécurité et de place
                # Path(csvfile).unlink()

            if notification:
                logger.info("Envoie de la notification mail")
                # Envoi de la notification mail ...
                message = (
                    "Synchronisation du flux agent sur le serveur gpuc2024\n"
                    "Voir le fichier de log disponible en pièce jointe \n"
                    "+ d'information sur le serveur christophe2022.univ.run \n\n"
                )

                mailsender = MailSender(
                    smtp_server=MAIL["SMTP_HOST"],
                    smtp_port=MAIL["SMTP_PORT"],
                    login_auth=MAIL["SMTP_LOGIN"],
                    password_auth=MAIL["SMTP_PASSWORD"],
                    sender_email=MAIL["SENDER"],
                )

                mailsender.send(
                    dest_mail=GPUC_MAIL_DEST,
                    subject="[GPUC][Notification][Pyetl]",
                    message=message,
                    file_attachement=[f"{TMP}/{GPUC_LOG}_{DATE_DU_JOUR}.log"],
                )
            return True
        return False

    @staticmethod
    def build_treeimg(*, tag: DataSource = DataSource.LDAP) -> bool:
        """Construit un fichier png représentant l'arbre hierarchique des structures

        Args:
            tag (DataSource): Quel sera la source de donnée. Valeur par défaut 'ldap'

        Returns:
            Booléen. True, le fichier a bien été généré. False sinon
        """
        gpstruct = Gpuc._get_structure(tag)

        if gpstruct:
            filename = f"{GPUC_STRUCTURE}/structure_{tag.value}_{DATE_DU_JOUR}"
            gpstruct.generate_graph(filename)
            logger.debug("Le fichier a été généré")
            return True
        return False
