import logging
import re
import sys
import time
from datetime import datetime, timezone

from ldappy.core.lib.ldapur import LdapUr
from notifire.mail.mailsender import MailSender
from pydantic import BaseModel, Field
from pyetl.config.global_settings import DATE_DU_JOUR, LDHAM_LOG, MAIL, SYNCHRO_MAIL_DEST, TMP
from wssiham.core.dossier_agent.modifDonneesPersonnelles import ModifDonneesPersonnelles
from wssiham.core.siham_ws_ldap.userSiham import UserSiham

logger = logging.getLogger(__name__)


class Ldham(BaseModel):
    """Connecteur ldap siham suivant le model ETL"""

    date_debut_observation: datetime
    date_fin_observation: datetime = Field(defaut=datetime.strptime("01/12/2999", "%d/%m/%Y").astimezone(timezone.utc))

    # Tache 1
    # Extraction des informations du LDAP
    def extract(self) -> list[tuple[str, str, str]]:
        """Retourne une liste contenant les informations suivantes dont la source est le ldap :
           - matricule = matricule SIHAM de l'agent (**runUnivSihamId**)
           - mail = mail professionnel (universitaire) de l'agent (**mail**)
           - login = login de connexion de l'agent (**supannAliasLogin**)

        Returns:
            Une liste de tuple contenant le matricule, le mail et le login de tous les agents de l'université
        """
        logger.info("Extraction des données agents du ldap")
        logger.info("Date de début d'observation : %s", self.date_debut_observation)
        logger.info("Date de fin d'observation : %s", self.date_fin_observation)
        result = []
        ldapur = LdapUr()
        data = ldapur.get_agents_mail_filter_by_date(
            date_debut_obsevation=self.date_debut_observation, date_fin_observation=self.date_fin_observation
        )
        for el in data:
            matricule = el["attributes"]["runUnivSihamId"][0]
            mail = el["attributes"]["mail"][0]
            login = el["attributes"]["supannAliasLogin"][0]
            result.append((matricule, mail, login))
        logger.info("Nombre d'agent récupérés : %s", len(result))
        ldapur.unbind()
        return result

    # Fonction : Activation de l'espace HRA
    @staticmethod
    def activation_hra(*, matricule: str, mail: str, login: str) -> bool:
        """Active l'espace SIHAM HRA pour un matricule.

        Args:
            matricule (str): matricule de l'agent
            mail (str): mail de l'agent
            login (str): login de l'agent

        Returns:
            Un booléen.
            True : L'activation de l'espace SIHAM HRA de l'agent a été réalisé avec succès
            False : L'activation de l'espace SIHAM HRA de l'agent a été interrompu

        """
        logger.debug("Information agent : (%s, %s, %s)", matricule, re.sub(r"[^@.]", "x", mail), login)
        # initialisation de wssiham
        user = UserSiham(lognid=login, matcle=matricule, selfsrv=True, adusml=mail)

        try:
            result = user.create_or_modify()
            if result:
                logger.debug("L'opération sur le matricule %s a été effectuée avec succès", matricule)
                activate = True
            else:
                logger.error("L'opération sur le matricule %s a été interrompu", matricule)
                activate = False
        except Exception as exc:
            logger.exception("Une erreur est survenue sur le connecteur webservice : %s", exc)
            sys.exit(1)
        return activate

    # Fonction : Ajout mailPro dans la section donnée personnelle de l'agent
    @staticmethod
    def ajout_mailpro_siham(*, matricule: str, mail: str) -> bool:
        """

        Args:
            matricule (str): matricule de l'agent
            mail (str): mail de l'agent

        Returns: True si l'opération d'ajout a été effectué avec succès, False sinon

        """
        logger.info("Initialisation de l'ajout du mail dans la fiche donnée personnelle du matricule %s", matricule)
        mdprs = ModifDonneesPersonnelles(matricule=matricule, type_action="A", type_numero="MPR", numero=mail)

        try:
            mdprs_response = mdprs.response()
            if mdprs_response.statut_maj.value == "1":
                logger.debug("L'ajout du mail pro sur le matricule %s a été effectué avec succès", matricule)
                success = True
            else:
                logger.error(
                    "L'opération sur le matricule %s a été interrompu. Erreur : %s",
                    matricule,
                    mdprs_response.statut_maj.value,
                )
                success = False
        except Exception:
            logger.exception("Une erreur est survenue sur le connecteur webservice")
            sys.exit(1)
        return success

    # Flux : Process d'activation des espaces HRA
    # 1) On récupère les agents du LDAP en fonction d'une date de début et d'une date de fin
    # 2) Pour chaque agent, on active son espace HRA via la méthode _active_hra
    # Bonus : On peut activer ou désactiver la notification mail
    def flux(self, notification: bool = False) -> None:
        """Process d'injection en masse des données mails et login dans SIHAM. Entraine l'activation de l'espace HRA de
        l'utilisateur.

        Args:
            notification (bool): Active ou non la notification mail
        """
        # Listes d'agents avec les données (matricule, mail, login)
        agents_ldap = self.extract()

        extra_data = {
            "nb_individus_ldap": len(agents_ldap),
            "activation_hra_ok": 0,
            "activation_hra_ko": 0,
            "ajout_mailpro_ok": 0,
            "ajout_mailpro_ko": 0,
        }

        if agents_ldap:
            # Pour chaque agent ...
            for agent in agents_ldap:
                # variable
                matricule = agent[0]
                mail = agent[1]
                login = agent[2]

                # ... activation de son espace hra
                activation = self.activation_hra(matricule=matricule, mail=mail, login=login)
                if activation:
                    extra_data["activation_hra_ok"] += 1
                else:
                    extra_data["activation_hra_ko"] += 1

                # #### pause 5 secondes entre les 2 webservices (par sécurité) #### #
                time.sleep(5)
                # ################################################################# #

                # ... ajout mail pro donnée personnelle
                add_mailpro = self.ajout_mailpro_siham(matricule=matricule, mail=mail)
                if add_mailpro:
                    extra_data["ajout_mailpro_ok"] += 1
                else:
                    extra_data["ajout_mailpro_ko"] += 1

                logger.info("Fin de la procédure pour le matricule %s", matricule)

            if notification:
                logger.info("Envoie de la notification mail")
                # Envoi de la notification mail ...
                objet = "injection ldap -> siham"
                message = (
                    f"Rapport ETL {datetime.now().strftime('%d/%m/%Y')} : Synchro login - mail\n\n"
                    f"Nombre d'agents extrait du ldap pour la période du "
                    f"{self.date_debut_observation.strftime('%d/%m/%Y')} au "
                    f"{self.date_fin_observation.strftime('%d/%m/%Y')} : {extra_data['nb_individus_ldap']}\n\n"
                    f"[ Webservice createOrModifyUserSiham ]\n"
                    f"Nombre d'espaces HRA activé : {extra_data['activation_hra_ok']}\n"
                    f"Nombre d'erreurs* : {extra_data['activation_hra_ko']}\n\n"
                    f"[ Webservice ModifDonneesPersonnelles ]\n"
                    f"Ajout des mails pro réussi : {extra_data['ajout_mailpro_ok']}\n"
                    f"Ajout des mails pro en échec** : {extra_data['ajout_mailpro_ko']}\n\n"
                    f"+ d'info dans le fichier de log dispo en pièce-jointe\n\n\n\n"
                    f"* Correspond à des agents non existant dans SIHAM ou déjà activés\n"
                    f"** Correspond à des agents qui possèdent déjà un mail pro 'MPR' "
                    f"ou à des agents non existants dans SIHAM\n\n"
                )

                mailsender = MailSender(
                    smtp_server=MAIL["SMTP_HOST"],
                    smtp_port=MAIL["SMTP_PORT"],
                    login_auth=MAIL["SMTP_LOGIN"],
                    password_auth=MAIL["SMTP_PASSWORD"],
                    sender_email=MAIL["SENDER"],
                )

                mailsender.send(
                    dest_mail=SYNCHRO_MAIL_DEST,
                    subject=f"[Siham][Notification][Pyetl] {objet}",
                    message=message,
                    file_attachement=[f"{TMP}/{LDHAM_LOG}_{DATE_DU_JOUR}.log"],
                )
        else:
            logger.info(
                "Aucun agent n'a été activé pour la période du s% au %s",
                self.date_debut_observation.strftime("%d/%m/%Y"),
                self.date_fin_observation.strftime("%d/%m/%Y"),
            )
