import logging
from datetime import datetime
from typing import Optional

from pyetl.config.global_settings import DATE_DU_JOUR, OUTPUT
from pyetl.config.ohris_settings import FLUX_ABSENCES_FILENAME, FLUX_UTILISATEURS_FILENAME, MOIS_PIVOT, SFTP_OHRIS
from pyetl.model.ohris.agent.compte import Compte, ListCompte
from pyetl.model.ohris.agent.conge import Conge, ListConge
from pyetl.tools.minidb import Ohrisdb
from pyetl.tools.sftp import SFTPClient
from pyetl.tools.sihamapi import ClientSihamApi
from pyetl.tools.toolbox.mcsv import Mcsv
from pyetl.tools.toolbox.mdate import Mdate
from pyetl.tools.toolbox.mxml import Mxml
from pyetl.tools.wssiham import WssihamTools
from siham_api_client.api.ohris import get_absences_agent, get_absences_agent_csv, get_agents, get_agents_xml
from siham_api_client.models.absence import Absence
from siham_api_client.models.ohris_agent import OhrisAgent
from siham_api_client.types import Response

logger = logging.getLogger(__name__)


class Ohris:
    @staticmethod
    def get_users_xml(matricules: list[str]) -> bytes:
        """Récupère les informations utilisateurs pour Ohris sous format XML en fonction d'une liste de matricule
        passé en paramètre

        ** Extract **

        Args:
            matricules (str): Matricule des agents dont les informations doivent être récupéré

        Returns:
            bytes sous format xml qui contient les informations des agents pour ohris.
        """
        client = ClientSihamApi.get_client()

        with client as c:
            response: Response[Response] = get_agents_xml.sync_detailed(client=c, body=matricules)
        return response.content

    @staticmethod
    def get_users_json(matricules: list[str]) -> list[OhrisAgent]:
        """Récupère les informations utilisateurs pour Ohris sous format json en fonction d'une liste de matricule
        passé en paramètre

        ** Extract **

        Args:
            matricules (str): Matricule des agents dont les informations doivent être récupéré

        Returns:
            Une liste de dictionnaire qui contient les informations des agents pour ohris.
        """
        client = ClientSihamApi.get_client()

        with client as c:
            return get_agents.sync(client=c, body=matricules)

    @staticmethod
    def get_abs_json(
        matricules: list[str], *, since: Optional[str] = None, include_managements: Optional[bool] = False
    ) -> list[Absence]:
        """Récupère les informations d'absences utilisateurs pour Ohris au format csv en fonction d'une liste
        de matricule passé en paramètre

        Args:
            matricules (str): Matricule des agents dont les informations doivent être récupéré
            since (str): Date charnière de la recherche (si null, aucune limitation sinon date à renseigner au format
                         ISO 8601 sans la précision des heures). Example : 2022-10-31
            include_managements (bool): Détermine si on inclut les absences à des fins de gestion

        Returns:
            Une liste de dictionnaire qui contienne les informations d'absences des agents recherchés
        """
        client = ClientSihamApi.get_client()
        with client as c:
            return get_absences_agent.sync(
                client=c, body=matricules, since=since, include_managements=include_managements
            )

    @staticmethod
    def get_abs_csv(matricules: list[str]) -> bytes:
        """Récupère les informations d'absences utilisateurs pour Ohris au format csv en fonction d'une liste
        de matricule passé en paramètre

        Args:
            matricules (str): Matricule des agents dont les informations doivent être récupéré

        Returns:
            bytes sous format csv qui contient les informations d'absences des agents pour ohris.
        """
        client = ClientSihamApi.get_client()
        with client as c:
            response: Response[Response] = get_absences_agent_csv.sync_detailed(client=c, body=matricules)
        return response.content

    @staticmethod
    def get_all_users_xml(start: Optional[str] = None, end: Optional[str] = None) -> bytes:
        """Récupère les informations de tous les utilisateurs pour Ohris sous format XML

        ** Extract **

        Returns:
            bytes format xml qui contient les informations des agents pour ohris passer en paramètre
        """
        fd, ed = "", ""

        if not start and not end:
            current_date = datetime.now()
            first_day_of_year = current_date.replace(month=1, day=1, hour=0, minute=0, second=0, microsecond=0)
            last_day_of_year = current_date.replace(month=12, day=31, hour=0, minute=0, second=0, microsecond=0)

            # Formatez les dates au format ISO 8601
            iso_format = "%Y-%m-%dT%H:%M:%S.%f"

            fd = f"{first_day_of_year.strftime(iso_format)[:-3]}+0400"
            ed = f"{last_day_of_year.strftime(iso_format)[:-3]}+0400"

        client = ClientSihamApi.get_client()
        with client as c:
            response: Response[Response] = get_agents_xml.sync_detailed(client=c, start=fd, end=ed, body=[])
        return response.content

    @classmethod
    def _builder(
        cls,
        matricules: list[str],
        guest: Optional[list[str]] = None,
        sftp: bool = False,
        prod: bool = False,
    ) -> bool:
        """Construit les fichiers xlm et csv pour les utilisateurs et les absences des agents pour Ohris.

        Args:
            matricules (str): Matricule des agents dont les informations doivent être récupéré
            guest (list[str]): Une liste de matricule siham à intégrer dans Ohris. Même s'ils ne sont pas dans les uos.
            sftp (bool): Push ou non les fichiers générés
            prod (bool): Push ou non sur le serveur de prod ou de recette de Ohris. Depends de sftp
        Returns:
            La construction des fichiers a été réalisée avec succès ou non
        """
        if matricules:
            if guest:
                matricules.extend(x for x in guest if x not in matricules)

            # ... récupère les données utilisateur via siham-api
            agt_ohris = cls.get_users_json(matricules)

            lohris = []
            # lohris = [Compte.from_users_dict(agent) for agent in agt_ohris]
            for agent in agt_ohris:
                if Compte.from_users_dict(agent):
                    lohris.append(Compte.from_users_dict(agent))
                else:
                    matricules.remove(agent.matricule)

            # ... récupère les données json absence via siham-api
            agt_abs = cls.get_abs_json(matricules, since=Mdate.get_date_n_months_ago(MOIS_PIVOT).strftime("%Y-%m-%d"))
            labs = [Conge.from_absence_dict(absence) for absence in agt_abs]

            if labs and lohris:
                # Initialisation des paramètres pour la sauvegarde
                xmlfilename = f"{FLUX_UTILISATEURS_FILENAME}_{DATE_DU_JOUR}.xml"
                csvfilename = f"{FLUX_ABSENCES_FILENAME}_{DATE_DU_JOUR}.csv"

                localxmlfile = f"{OUTPUT}/{xmlfilename}"
                localcsvfilename = f"{OUTPUT}/{csvfilename}"

                # Sauvegarde des fichiers
                # ... fichier absence au format csv
                ListConge(agents_absence=labs).to_csv(localcsvfilename)

                # ... fichier utilisateur au format xml
                ListCompte(agents_compte=lohris).to_xml(localxmlfile)

                if sftp:
                    path_ohris = SFTP_OHRIS["PATH_FPROD"] if prod else SFTP_OHRIS["PATH_FOTHER"]
                    # ... envoie des fichiers sur le serveur sftp
                    sftp_client = SFTPClient(
                        SFTP_OHRIS["SFTP_HOST"],
                        SFTP_OHRIS["SFTP_PORT"],
                        SFTP_OHRIS["SFTP_USERNAME"],
                        SFTP_OHRIS["SFTP_PASSWORD"],
                    )
                    sftp_client.connect()
                    sftp_client.upload_file(f"{path_ohris}/{xmlfilename}", localxmlfile)
                    logger.debug("Envoie du fichier xml via sftp dans le dossier")
                    sftp_client.upload_file(f"{path_ohris}/{csvfilename}", localcsvfilename)
                    logger.debug("Envoie du fichier csv via sftp")
                    sftp_client.disconnect()

                    # Suppression des fichiers locaux après envoie par souci de sécurité et de confidentialité
                    # Path(localxmlfile).unlink()
                    # Path(localcsvfilename).unlink()

                return True
            logger.debug("Les données utilisateurs ou absences sont vides")
            logger.debug("Les fichiers xml et csv n'ont pas été sauvegardé")
            return False
        logger.error("Une erreur est survenue lors de la récupération des matricules via le WS SIHAM listeAgent")
        return False

    @classmethod
    def flux_restricted_perimeter(cls, uos: list[str]) -> bool:
        """Flux permettant de créer les fichiers utilisateurs et absences en fonction des agents affectés dans des uos
        pour Ohris.
        Les données absences et utilisateurs récupérées sont préformatées dans le format demandé par ohris, càd :
        - format csv pour les données absences
        - format xlm pour les donéees utilisateurs

        Args:
            uos (list[str]): Liste de UO. Par défault, uo dsi et drh

        Returns:
            Booléen. True, le flux s'est bien déroulé. False sinon
        """
        ohrisdb = Ohrisdb(name="agents-restreint")

        # On récupère les matricules des agents concernés par le test via le ws siham liste-agent
        matricules_ws = WssihamTools.get_agents_uo(uos)

        if matricules_ws:
            # ... on met à jour la minidb avec les matricules
            # /!\ Ne sert à rien pour le moment
            ohrisdb.update_matricules(matricules=matricules_ws)

            # ... récupère le string xml via siham-api
            utilisateurs_xml = cls.get_users_xml(matricules_ws)

            # ... récupère le string csv via siham-api
            abs_csv = cls.get_abs_csv(matricules_ws)

            if utilisateurs_xml and abs_csv:
                logger.debug("xml et csv récupérés via siham-api")

                # Initialisation des paramètres pour la sauvegarde
                xmlfilename = f"{OUTPUT}/{FLUX_UTILISATEURS_FILENAME}_{DATE_DU_JOUR}.xml"
                csvfilename = f"{OUTPUT}/{FLUX_ABSENCES_FILENAME}_{DATE_DU_JOUR}.csv"

                # ... sauvegarde des fichiers
                Mxml.sxml_to_file(utilisateurs_xml, outputfile=xmlfilename)
                Mcsv.csv_to_file(abs_csv, outputfile=csvfilename)

            else:
                logger.debug("Les données utilisateurs ou absences sont vides")
                logger.debug("Les fichiers xml et csv n'ont pas été sauvegardé")
                return False
        else:
            logger.error("Une erreur est survenue lors de la récupération des matricules via le WS SIHAM listeAgent")
            return False

        return True

    @classmethod
    def flux_restricted_perimeter_custom(
        cls,
        uos: list[str],
        guest: Optional[list[str]] = None,
        sftp: bool = False,
        prod: bool = False,
    ) -> bool:
        """Flux permettant de créer les fichiers utilisateurs et absences en fonction des agents affectés dans des uos
        pour Ohris.
        Les données sont récupérées sous format json et construit à partir du modèle ohris.agent

        Args:
            uos (list[str]): Liste de UO. Par défault, uo dsi et drh
            guest (list[str]): Une liste de matricule siham à intégrer dans Ohris. Même s'ils ne sont pas dans les uos.
            sftp (bool): Push ou non les fichiers générés
            prod (bool): Push ou non sur le serveur de prod ou de recette de Ohris. Depends de sftp

        Returns:
            Booléen. True, le flux s'est bien déroulé. False sinon
        """
        # ohrisdb = Ohrisdb(name='agents-restreint')

        # On récupère les matricules des agents concernés par le test via le ws siham liste-agent
        matricules_ws = WssihamTools.get_agents_uo(uos)

        return cls._builder(matricules_ws, guest, sftp, prod)

    @classmethod
    def flux_alluniv(
        cls,
        guest: Optional[list[str]] = None,
        sftp: bool = False,
        prod: bool = False,
    ) -> bool:
        """Flux permettant de créer les fichiers utilisateurs et absences de tous les agents biatss affectés
        à l'université de la Réunion pour Ohris.
        Les données sont récupérées sous format json et construit à partir du modèle ohris.agent

        Args:
            guest (list[str]): Une liste de matricule siham à intégrer dans Ohris. Même s'ils ne sont pas dans les uos.
            sftp (bool): Push ou non les fichiers générés
            prod (bool): Push ou non sur le serveur de prod ou de recette de Ohris. Depends de sftp

        Returns:
            Booléen. True, le flux s'est bien déroulé. False sinon
        """
        # On récupère les titulaires BIATSS via le ws siham liste-agent
        titubiatss_ws = WssihamTools.get_titubiatss()

        # On récupère les contractuels BIATSS via le ws siham liste-agent
        contractbiatss_ws = WssihamTools.get_contractbiatss()

        # On merge les deux listes en supprimant les doublons
        matricules_ws = list(set(titubiatss_ws + contractbiatss_ws))

        return cls._builder(matricules_ws, guest, sftp, prod)
