import logging

from ldappy.core.lib.ldapur import LdapUr
from notifire.mail.mailsender import MailSender
from pyetl.config.global_settings import DATE_DU_JOUR, DEFAUT_MAIL_DEST, MAIL, SIDAP_LOG, TMP
from pyetl.tools.sihamapi import ClientSihamApi
from siham_api_client.api.habilitations import get_all_permissions
from siham_api_client.models.agent_permission import AgentPermission

logger = logging.getLogger(__name__)


class Sidap:
    """Connecteur Siham -> Ldap"""

    @staticmethod
    def ldap_agent_hab(ldapur: LdapUr) -> list[tuple[str, str]]:
        """Liste les agents dans le référentiel LDAP qui possède un accès à SIHAM HRA
        Càd : runUnivAuthorization = siham-hra:*

        ** Extraction **

        Returns: Une liste de tuple contenant le matricule siham de l'agent ainsi que son statut d'habilitation

        """
        result = []

        data = ldapur.recup_agent_habsiham()
        logger.debug("Récupération des utilisateurs possédants un 'runUnivAuthorization = siham-hra:*'")
        for el in data:
            spew = "siham-hra"
            matricule = el["attributes"]["runUnivSihamId"][0]
            habsiham = [e for e in el["attributes"]["runUnivAuthorization"] if spew in e]

            result.append((matricule, habsiham[0].split(":")[1]))

        logger.debug("Nombre d'agents possédant une habilitation SIHAM dans le ldap : %s", len(result))
        return result

    @staticmethod
    def sihamapi_habilitation() -> list[str]:
        """Récupère une liste de tuple de matricule SIHAM ainsi que leur profil d'habilitation associé.
        Ces matricules (agents) disposent d'un accès administratif à l'interface HRA de SIHAM

        ADMIN : Accès à toute l'interface SIHAM HRA en lecture et en écriture
        LECTURE : Accès à toute l'interface SIHAM HRA en lecture
        RA : Responsable Administratif
        GC : Gestionnaire de Composante

        ** Extraction **

        Returns:
            Une liste de tuple comprenant le matricule SIHAM ainsi que leur profil d'habilitation associé
        """
        result = []

        client = ClientSihamApi.get_client()
        with client as c:
            api_response: AgentPermission = get_all_permissions.sync(client=c, all_=False)

        # return api_response
        for data in api_response:
            agents = data.agent.identity.matricule
            if len(data.permissions) >= 3:
                permission = "GC"
            elif len(data.permissions) == 2:
                permission = "RA"
            elif len(data.permissions) == 1:
                if data.permissions[0].code == "AMUEALLHLO":
                    permission = "ADMIN"
                elif data.permissions[0].code == "URALLLEC":
                    permission = "LECTURE"
            else:
                permission = "UNK"
                logger.debug("habilitation inconnue")

            result.append((agents, permission))

        logger.debug("Nombre d'agents possédant une habilitation dans SIHAM [SIHAMAPI] : %s", len(result))
        return result

    @staticmethod
    def process(l1: list[tuple], l2: list[tuple], ldap: LdapUr, *, notification: bool = False) -> bool:
        """Function principale pour la mise à jour des runUnivAuthorisations des agents
        Si l'agent possède une habilitation dans SIHAM mais qu'il n'y a pas de siham-hra dans le LDAP
            alors ajout de la valeur adéquat dans l'attribut runUnivAuthorization de l'agent sur le LDAP.
        Si l'agent possède une habilitation différente dans SIHAM et dans le LDAP
            alors mise à jour de la valeur runUnivAuthorization de l'agent
            (suppression ancienne valeur + ajout nouvelle)
        Si l'habilitation de l'agent existe dans le ldap mais plus dans siham-api
            alors suppression de runUnivAuthorization = siham-hra*

        ** Transform **

        Args:
            l1 (list[tuple]): Liste des agents extraite de siham-api.
            l2 (list[tuple]): Liste des agents extraite de Ldap
            notification (bool) : Active ou désactive l'envoi de notification par mail
            ldap: (LdapUr)
        """

        def multiadd() -> None:
            """Function pour ajouter en masse les authorizations sur le ldap pour les agents qui possèdent une
            authorization dans siham hra
            """
            # On récupère uniqument les matricules qui sont dans l1 (agent siham-api) et pas dans l2 (ldap)
            # diff_asa = [set([item[0] for item in l1]) - set([item[0] for item in l2])]
            diff_asa = [{item[0] for item in l1} - {item[0] for item in l2}]

            logger.debug("Matricules ne possédant pas d'attribut runUnivAuthorization=siham-hra:* : %s", diff_asa)
            # Ajout des attributs runUnivAuthorization qui ne sont pas dans le ldap
            if diff_asa:
                for matricule in diff_asa:
                    etiquette = next((j for i, j in l1 if i == matricule))
                    result = ldap.add_authorization_by_matricule(f"siham-hra:{etiquette.lower()}", matricule=matricule)
                    if result["success"]:
                        logger.debug("%s", result["debug"])
                    else:
                        logger.debug("%s", result["debug"])

        def multidel() -> None:
            """Fucntion pour supprimer en masse les authorization sur le ldap
            pour les agents qui ont siham-hra sur le ldap mais qui n'ont pas une habilitation dans siham
            """
            # On récupère uniquement les matricules qui sont dans l2 (ldap)et pas dans l1 (agent siham-api)
            diff_ldap = [{item[0] for item in l2} - {item[0] for item in l1}]

            logger.debug(
                "Matricules qui doivent être supprimer car n'ont plus d'habilitation sur SIHAM HRA : %s", diff_ldap
            )
            # Suppressions des valeurs attributs runUnivAuthorization qui sont dans le ldap mais pas dans siham-api
            if diff_ldap:
                for matricule in diff_ldap:
                    etiquette = next((j for i, j in l2 if i == matricule))
                    result = ldap.delete_authorization_by_matricule(
                        f"siham-hra:{etiquette.lower()}", matricule=matricule
                    )
                    if result["success"]:
                        logger.debug("%s", result["debug"])
                    else:
                        logger.debug("%s", result["debug"])

        def multimod() -> None:
            """Function pour modifier en masse les authorization sur le ldap pour les agents dont les habilitations sont
            différents dans ldap et siham
            """
            # diff_hab = [v1 for i1, v1 in l1 for i2, v2 in l2 if (i1 == i2) and (v1 == v2)]
            for i, j in l1:
                for k, v in l2:
                    # S'il y a une différence entre l'authorisation de siham et ldap
                    if (i == k) and (j.lower() != v.lower()):
                        logger.debug("matricule %s -- siham-api authorization %s -- ldap authorization %s", i, j, v)
                        rdel = ldap.delete_authorization_by_matricule(f"siham-hra:{v.lower()}", matricule=k)
                        if rdel["success"]:
                            logger.debug("%s", rdel["debug"])
                            radd = ldap.add_authorization_by_matricule(f"siham-hra:{j.lower()}", matricule=i)
                            if radd["success"]:
                                logger.debug("%s", radd["debug"])
                            else:
                                logger.debug("%s", radd["debug"])
                        else:
                            logger.debug("%s", rdel["debug"])

        if l1 and l2:
            logger.debug("Début des opérations sur le ldap")
            multiadd()
            multidel()
            multimod()
            logger.info("Les opérations ont été réalisés avec succès !!")

            if notification:
                mailsender = MailSender(
                    smtp_server=MAIL["SMTP_HOST"],
                    smtp_port=MAIL["SMTP_PORT"],
                    login_auth=MAIL["SMTP_LOGIN"],
                    password_auth=MAIL["SMTP_PASSWORD"],
                    sender_email=MAIL["SENDER"],
                )

                mailsender.send(
                    dest_mail=DEFAUT_MAIL_DEST,
                    subject="[Ldap][Notification][Pyetl] Synchro Habilitations SIHAM",
                    message="",
                    file_attachement=[f"{TMP}/{SIDAP_LOG}_{DATE_DU_JOUR}.log"],
                )
            return True

        logger.error("Une erreur est survenue")
        return False

    def flux(self, notification: bool = False) -> bool:
        """Flux pour synchro toutes les habilitations SIHAM des agents dans le Ldap

        Args:
            notification (bool): Active ou désactive la notification mail

        Returns: Succès ou non de la synchro

        """
        ldap = LdapUr()
        r_ldap = self.ldap_agent_hab(ldap)
        r_siham = self.sihamapi_habilitation()

        # Si le ldap ou sihamapi ne renvoi rien
        if not r_ldap or not r_siham:
            logger.error("Aucune donnée récupérer dans ldap et/ou siham")
        else:
            p = self.process(r_siham, r_ldap, ldap, notification=notification)
        ldap.unbind()

        return p
