import logging
from dataclasses import dataclass, field
from datetime import datetime
from pathlib import Path
from typing import Any, Optional, Union

from pyetl.config.global_settings import MINIDB, TABLE_OHRIS, TABLE_SIHAMAPI
from tinydb import Query, TinyDB

logger = logging.getLogger(__name__)


@dataclass
class Minidb:
    """Boite à outils pour des besoins de stockage de donnée"""

    file: str = MINIDB
    table_name: Optional[str] = field(default=None)
    cdb: TinyDB = field(init=False, repr=True)
    ctable: TinyDB.table = field(init=False, repr=True)

    def __post_init__(self):
        if Path(self.file).exists():
            self.cdb = TinyDB(str(self.file))
            if self.table_name:
                self.ctable = self.cdb.table(self.table_name)


@dataclass
class TokenDb(Minidb):
    """Classe de base pour la gestion des tokens JWT"""

    table_name = str

    def __post_init__(self):
        super().__post_init__()

    def get_token(self, user: str) -> Optional[str]:
        s = self.ctable.search(Query().user == user)
        return s[0]["token"] if s else None

    def create_schema(self, *, user: str = "sda", token: str = "") -> None:
        self.ctable.insert({"user": user, "token": token, "date_modification": datetime.now().isoformat()})

    def update_token(self, *, user: str = "sda", token: str) -> None:
        if not self.ctable.all():
            logger.debug("Le fichier minidb.json est vide. Initialisation ...")
            self.create_schema(user=user, token=token)
        else:
            self.ctable.update({"token": token, "date_modification": datetime.now().isoformat()}, Query().user == user)

    def get_conninfo(self, user: str = "sda") -> Optional[tuple[str, Any, Any]]:
        s = self.ctable.search(Query().user == user)
        return (user, s[0]["token"], s[0]["date_modification"]) if s else (None, None, None)


@dataclass
class Sihamapidb(TokenDb):
    """Classe pour la gestion des tokens JWT pour siham-api"""

    def __post_init__(self):
        self.table_name = TABLE_SIHAMAPI
        super().__post_init__()


@dataclass
class Ohrisdb(Minidb):
    name: str = "agents"

    def __post_init__(self):
        self.table_name = TABLE_OHRIS
        super().__post_init__()

    def get_matricules(self) -> list[Union[str, None]]:
        """Récupère les matricules

        Returns:
            Les matricules pour la définition des fichiers pour Ohris
        """
        s = self.ctable.search(Query().name == self.name)
        if s:
            return s[0]["matricules"]
        return []

    def create_schema(self, *, matricules: Optional[list[Union[str, None]]] = None) -> None:
        """Crée la table siham-api et ajoute le model associé

        Args:
            matricules (list[str]): Liste de matricule qui a été extraite de siham
        """
        self.ctable.insert(
            {"name": self.name, "matricules": matricules, "date_modification": datetime.now().isoformat()}
        )

    def update_matricules(self, *, matricules: list[Union[str, None]]) -> None:
        """Mets à jour le token jwt associé à l'user dans la table siham_api

        Args:
            matricules (list[str]): Liste de matricule qui va serv
        """
        if not self.ctable.all():
            logger.debug("Le fichier minidb.json est vide. Initialisation ...")
            self.create_schema(matricules=matricules)
        else:
            # Todo : mettre en place l'ajout uniquement des nouveaux matricules.
            # Récupère les précédents matricules stockés dans la miniddb
            # previous_matricules = self.get_matricules()
            # Mets à jour la liste
            # updated_matricules = previous_matricules.extend(matricules)

            self.ctable.update(
                {"matricules": matricules, "date_modification": datetime.now().isoformat()}, Query().name == self.name
            )
