import logging
from dataclasses import dataclass, field
from pathlib import Path
from typing import Optional

import paramiko

logger = logging.getLogger(__name__)


@dataclass
class SFTPClient:
    hostname: str
    port: int
    username: str
    password: str
    private_key_path: Optional[str] = None
    ssh_client: paramiko.client.SSHClient = field(init=False, repr=False)

    def __post_init__(self):
        self.ssh_client = paramiko.SSHClient()

    def connect(self):
        self.ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        if self.private_key_path:
            private_key = paramiko.RSAKey(filename=self.private_key_path)
            self.ssh_client.connect(
                hostname=self.hostname,
                port=self.port,
                username=self.username,
                pkey=private_key,
                look_for_keys=False,
            )
        else:
            self.ssh_client.connect(
                hostname=self.hostname,
                port=self.port,
                username=self.username,
                password=self.password,
                look_for_keys=False,
            )

    def disconnect(self):
        self.ssh_client.close()
        logger.debug("%s is disconnected to server %s:%s", self.username, self.hostname, self.port)

    def get_list_of_files(self, remote_file_path):
        sftp_client = self.ssh_client.open_sftp()
        try:
            logger.debug("lists of files %s", sftp_client.listdir(remote_file_path))
        except Exception:
            logger.exception("Error while getting the list of files")
            raise

    def download_file(self, remote_file_path):
        local_file_path = "temp" / Path(remote_file_path)
        logger.debug("downloading file %s to remote %s", remote_file_path, local_file_path)

        sftp_client = self.ssh_client.open_sftp()
        try:
            sftp_client.get(remote_file_path, local_file_path)
        except FileNotFoundError:
            logger.exception("File: %s was not found on the server", remote_file_path)
        sftp_client.close()

    def upload_file(self, remote_file_path, local_file_path):
        logger.debug("uploading file %s to remote %s", local_file_path, remote_file_path)

        sftp_client = self.ssh_client.open_sftp()
        try:
            sftp_client.put(local_file_path, remote_file_path)
        except FileNotFoundError:
            logger.exception("File %s was not found on the local system", local_file_path)
        sftp_client.close()

    def execute_command(self, command):
        stdin, stdout, stderr = self.ssh_client.exec_command(command)
        logger.debug(stdout.readlines())
        logger.debug(stderr.readlines())
