import logging
import os
import sys
from dataclasses import dataclass, field
from typing import Optional, Union

import jwt
from pyetl.tools.minidb import Sihamapidb
from siham_api_client import AuthenticatedClient, Client
from siham_api_client.api.authentification import authenticate
from siham_api_client.errors import UnexpectedStatus
from siham_api_client.models.security_input_authenticate import SecurityInputAuthenticate

logger = logging.getLogger(__name__)


@dataclass
class Connection:
    """Simple Connection
    Class pour la connection simplifiée à siham-api.

    Notes:
        http://api2023.univ.run:8080/siham/api/swagger/
    """

    host: str

    user: str = field(default="")
    password: str = field(default="", repr=False)
    token: str = field(default="", repr=False)
    client: Optional[AuthenticatedClient] = field(default=None, repr=False)

    def __post_init__(self):
        if not self.token or not self._validtokenjwt(self.token):
            self.token = self._tokenjwt()

        self.client = AuthenticatedClient(base_url=self.host, token=self.token)
        logger.info("Initalisation du client openapi")

    def _tokenjwt(self) -> Union[str, None]:
        """Récupère le token jwt de siham-api

        Returns:
            Token jwt siham-api sous forme de string
        """
        tok = ""
        api_client = Client(base_url=self.host)
        with api_client as client:
            try:
                my_data: SecurityInputAuthenticate = authenticate.sync(
                    client=client, body=SecurityInputAuthenticate(self.user, self.password)
                )
                tok = my_data.token
            except UnexpectedStatus:
                logger.exception("Exception when calling SecurityInputAuthenticate")
                sys.exit(1)
        return tok

    @staticmethod
    def _validtokenjwt(token: str) -> bool:
        """Vérifie la validité d'un token jwt en fonction de sa date d'expiration

        Args:
            token (str): token jwt

        Returns:
            True : le token est valide
            False : le token n'est pas valide
        """
        validtok = False
        try:
            data = jwt.decode(token, algorithms=["RS256"], options={"verify_exp": True, "verify_signature": False})
            if data:
                validtok = True
        except jwt.ExpiredSignatureError:
            validtok = False
        return validtok


class ClientSihamApi:
    """Client spécifique pour siham-api
    http://api2023.univ.run:8080/siham/api/swagger/
    """

    @staticmethod
    def get_client() -> Connection.client:
        """Récupère un client spécifique pour être utiliser avec siham-api

        Returns:
            Un AuthenticatedClient qui sera utilisé comme client siham-api
        """
        db = Sihamapidb()
        token = db.get_token(os.environ.get("API2023_USER"))

        conn = Connection(
            host=os.environ.get("API2023_HOST"),
            user=os.environ.get("API2023_USER"),
            password=os.environ.get("API2023_PASS"),
            token=token,
        )

        if token == conn.token:
            logger.debug("Le token JWT est toujours valide")
        else:
            logger.debug("Mise à jour du token JWT")
            db.update_token(user=os.environ.get("API2023_USER"), token=conn.token)

        return conn.client
