from datetime import datetime
from typing import Union

from pyetl.model.siham.status import SousCategorie
from wssiham.core.liste_agents.recupListeAgents import RecupListeAgents
from wssiham.model.agent.informationAgent import InformationAgentLA
from wssiham.model.parametre.agent import ParamAffectationUO, ParamPIP, ParamSpecialitesNiv1Agent
from wssiham.model.response.liste_agents.genericAgentsReponse import InformationAgentList


class WssihamTools:
    @staticmethod
    def get_agents_uo(uos: list[str], date_observation: str = datetime.now().strftime("%d/%m/%Y")) -> list[str]:
        """Récupère une liste de matricule à partir du webservice recupListeAgent en fonction des codes UO SIHAM

        Args:
            date_observation (str): date d'observation pour la récupération des agents au format dd/mm/YYYY.
                                    Par défaut, date du jour
            uos (list[str]): liste de code UO SIHAM

        Returns:
            Une liste de matricule qui correspond aux agents appartenans aux UO recherchés
        """
        paramaff = [
            ParamAffectationUO(codeUO=uo, temoinUOFille="1", typeLienAffectationUO="HIE").model_dump() for uo in uos
        ]

        rla = RecupListeAgents(date_observation=date_observation, tem_etat="A", liste_affectation_uo=paramaff)
        response: Union[list, InformationAgentList] = rla.response()
        if response:
            tmp: list[InformationAgentLA] = response.information_agent
            return [infoa.matricule for infoa in tmp]
        return []

    @staticmethod
    def get_titubiatss(date_observation: str = datetime.now().strftime("%d/%m/%Y")) -> list[str]:
        """Récupère une liste de matricule à partir du webservice recupListeAgent
        en fonction du type de spécialité (BAP)

        Args:
            date_observation (str): date d'observation pour la récupération des agents au format dd/mm/YYYY.
                                    Par défaut, date du jour

        Returns:
            Une liste de matricule qui correspond aux agents appartenans au type de spécialité BAP (BIATSS).
            C'est-à-dire au titulaire BIATSS.
        """
        paramaff = [
            ParamAffectationUO(codeUO="HU00000000", temoinUOFille="1", typeLienAffectationUO="HIE").model_dump()
        ]
        paramspe = [ParamSpecialitesNiv1Agent(codeTypeSpecialiteAgent="BAP").model_dump()]

        rla = RecupListeAgents(
            date_observation=date_observation,
            tem_etat="A",
            liste_affectation_uo=paramaff,
            specialites_niv1_agent=paramspe,
        )
        response: Union[list, InformationAgentList] = rla.response()
        if response:
            tmp: list[InformationAgentLA] = response.information_agent
            return [infoa.matricule for infoa in tmp]
        return []

    @staticmethod
    def get_contractbiatss(date_observation: str = datetime.now().strftime("%d/%m/%Y")) -> list[str]:
        """Récupère une liste de matricule à partir du webservice recupListeAgent en fonction de code PIP qui
        correspond au contractuel BIATSS

        Args:
            date_observation (str): date d'observation pour la récupération des agents au format dd/mm/YYYY.
                                    Par défaut, date du jour

        Returns:
            Une liste de matricule qui correspond aux agents contractuels BIATSS
        """
        paramaff = [
            ParamAffectationUO(codeUO="HU00000000", temoinUOFille="1", typeLienAffectationUO="HIE").model_dump()
        ]
        parampip = [ParamPIP(codePIP=codepip).model_dump() for codepip in SousCategorie.BIATSS.value]

        rla = RecupListeAgents(
            date_observation=date_observation, tem_etat="A", liste_affectation_uo=paramaff, liste_pip=parampip
        )
        response: Union[list, InformationAgentList] = rla.response()
        if response:
            tmp: list[InformationAgentLA] = response.information_agent
            return [infoa.matricule for infoa in tmp]
        return []
